=head1 NAME

TextPipe - Text processing pipe

=head1 SYNOPSIS

use TextPipe;

my $inputText = "some beautifully marked-up text";
my $outputText = TextPipe::process(
    '/tmp/textpipe',
    'rest2html',
    $inputText);

=head1 DESCRIPTION

Pipes text file through a user-defined external text processing tool

=head1 VERSION

0.1 - 2005-01-29

=head1 AUTHOR

Andrea Rota <a@xelera.it>, http://www.xelera.it/

=head1 BUGS

Only allows text processing tools which take options followed by either
``inputFile [outputFile]`` or ``outputFile inputFile`` (the latter can actually
be used, e.g., this way: ``command -x -y -z -o outputFile inputFile``).

=head1 LICENSE

This is free software with ABSOLUTELY NO WARRANTY under the terms of the GNU
General Public License version 2 or, at your option, any later version.

=cut

package TextPipe;

use strict;
use warnings;

use vars qw($VERSION $DEBUG);

$VERSION = '0.1';
$VERSION = eval $VERSION;

$DEBUG = 0;

sub process {
    my ($spool_path, $processor_cmdline, $input_text, $outputfile_extension) = @_;
    
    my $output_text;
    
    eval {
        # append a '/' if $spool_path is not void and does not already end with
        # a '/'
        $spool_path .= '/' if ($spool_path ne '' and $spool_path !~ /\/$/);
        
        # create spool directory, if necessary
        qx(mkdir -p $spool_path) if (!-d $spool_path);
        
        # put body text in a temp file so that the text processing command
        # can access it
        my $filename = $spool_path . random_filename();
        open FH, ">$filename";
        print FH $input_text;
        close FH;
    
        # let the text processing command format text
        if($outputfile_extension) {
            qx($processor_cmdline $filename $filename$outputfile_extension");
            open IN, "<$filename$outputfile_extension";
            $output_text .= $_ while(<STDIN>);
            close IN;
            unlink $filename . $outputfile_extension unless $DEBUG;
        } else {
            print STDERR "$processor_cmdline $filename" if $DEBUG;
            $output_text = qx($processor_cmdline $filename);
        }

        # delete the temp file
        unlink $filename;
  };

  return $output_text;
}

sub random_filename {
    my @chars=('a'..'z','A'..'Z','0'..'9','_');
    my $filename;
    foreach (1..64) { $filename.=$chars[rand @chars]; }
    return $filename;
}

1;
